/* This file was generated by syncqt. */
#ifndef QT_QT3DLOGIC_VERSION_H
#define QT_QT3DLOGIC_VERSION_H

#define QT3DLOGIC_VERSION_STR "5.15.9"

#define QT3DLOGIC_VERSION 0x050F09

#endif // QT_QT3DLOGIC_VERSION_H
